SystemOrganization addCategory: #'Rest-Server-Handlers'!
SystemOrganization addCategory: #'Rest-Server-Services'!

!Collection methodsFor: '*rest-server' stamp: 'cs 2/16/2009 00:38'!
jsonWriteOn: aStream
	| needComma |
	needComma := false.
	aStream nextPut: $[.
	self do: [:v |
		needComma
			ifTrue: [ aStream nextPutAll: ', ' ]
			ifFalse: [ needComma := true ].
		v jsonWriteOn: aStream.
	].
	aStream nextPut: $].! !

Resource subclass: #RestHttpService
	instanceVariableNames: 'handlers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Services'!

RestHttpService class
	instanceVariableNames: 'site resource'!

!RestHttpService commentStamp: 'mt 2/16/2009 14:56' prior: 0!
I am a restful http service. I am primarily used for the XP integration platform project. (Package: 'Xpf')!

RestHttpService class
	instanceVariableNames: 'site resource'!

!RestHttpService class methodsFor: 'service' stamp: 'mt 2/19/2009 16:05'!
isRunning
	^site notNil and: [site isServing]! !

!RestHttpService class methodsFor: 'service' stamp: 'cs 2/19/2009 17:51'!
port
	^site port! !

!RestHttpService class methodsFor: 'service' stamp: 'mt 2/19/2009 17:15'!
service
	^resource! !

!RestHttpService class methodsFor: 'service' stamp: 'mt 2/19/2009 16:08'!
start

	self startOn: 8082.

! !

!RestHttpService class methodsFor: 'service' stamp: 'cs 2/19/2009 17:59'!
startOn: aPort
	"Start exactly one http service. Does nothing if server is already running."

	self isRunning ifTrue: [^self].
	site := Site new
		name: 'rest';
		host: 'localhost' ip: '127.0.0.1' port: aPort;
		yourself.

	SwazooServer singleton addSite: site.
	
	resource := RestHttpService new uriPattern: '/'.
	site
		"start;"
		addResource: resource.
	SwazooServer singleton startSite: 'rest'.! !

!RestHttpService class methodsFor: 'service' stamp: 'mt 2/19/2009 17:15'!
stop
	"Shutdown the server. Does nothing if server is not running."

	site ifNotNil:
		[site stop.
		SwazooServer singleton removeSite: site.
		site := nil.
		resource := nil].! !

!RestHttpService methodsFor: 'serving' stamp: 'cs 2/19/2009 17:49'!
answerTo: request
	
	^ (handlers
		detect: [:ass| request uri asString startsWith: ass key]
		ifNone: [^ HTTPResponse notFound])
			value new process: request! !

!RestHttpService methodsFor: 'initialization' stamp: 'mt 2/2/2009 10:12'!
initialize
	super initialize.
	handlers := OrderedCollection new	"needed for priorities"! !

!RestHttpService methodsFor: 'serving' stamp: 'cs 2/15/2009 16:07'!
installHandler: aRequestHandlerClass on: aPathString
	(handlers anySatisfy: [:ass | ass key = aPathString ])
		ifTrue: [^ false].
	handlers add: aPathString -> aRequestHandlerClass! !

Object subclass: #RestHttpHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestHttpHandler methodsFor: 'request processing' stamp: 'mt 2/16/2009 15:18'!
process: anHttpRequest
	"Handles an http request. Should answer with an http response."
	
	self subclassResponsibility! !

RestHttpHandler subclass: #RestPerformHttpHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestPerformHttpHandler methodsFor: 'request processing' stamp: 'cs 2/19/2009 17:51'!
process: anHTTPRequest
	"Takes first element in path as classname and sends message from third path element to that class."

	|pc class selector result|
	pc := anHTTPRequest uri identifierPath.
	class := pc second asSymbol.
	selector := pc third asSymbol.
	
	Smalltalk at: class ifPresent: [:c | 
		(c class allSelectors includes: selector) ifTrue: [
			result := c perform: selector]].

	^ HTTPResponse ok
		entity: result asString;
		yourself! !

RestHttpHandler subclass: #RestResourceHttpHandler
	instanceVariableNames: 'resources'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestResourceHttpHandler methodsFor: 'http-post-helper' stamp: 'cs 2/19/2009 21:14'!
create: resClass from: fields
	"Works only with a new project or a new user."
	
	| entity |
	entity := resClass new.
	fields keysAndValuesDo: [:key :value |
		entity instVarNamed: key asString put: value first].
	resClass perform: ('add', (self resolveName: resClass) capitalized, ':') asSymbol with: entity.
	^ HTTPResponse ok entity: entity id asString! !

!RestResourceHttpHandler methodsFor: 'http-delete-helper' stamp: 'cs 2/19/2009 21:14'!
delete: resClass id: id
	"Delete a resource with the specified identifier. Works only with projects and users."

	| entity |
	entity := resClass all
		detect: [:each | each id = id]
		ifNone: [^ self responseForNotFound: resClass id: id].
	resClass
		perform: ('delete', (self resolveName: resClass) capitalized, ':') asSymbol
		with: entity.
	^ HTTPResponse ok entity: 'deleted'.! !

!RestResourceHttpHandler methodsFor: 'private' stamp: 'test 2/19/2009 21:12'!
getRequestChainFor: aPathComponentList
	"Create a list of associations for the request. That list contains resource classes and their requested ids."
	
	| chain resClass idNumber |
	chain := OrderedCollection new.
	aPathComponentList size even ifTrue: [aPathComponentList add: 'nil'].
	(aPathComponentList atAll: (2 to: aPathComponentList size)) pairsDo: [:resName :idName | 
		resClass := self resolveResource: resName ifAbsent: [nil].
		idName = 'nil'
			ifTrue: [idNumber := nil]
			ifFalse: [idNumber := idName asNumber].
		resClass ifNotNil: [chain add: (resClass -> idNumber)]].
	^ chain! !

!RestResourceHttpHandler methodsFor: 'initialization' stamp: 'mt 2/18/2009 14:57'!
initialize
	
	super initialize.
	resources := Dictionary new! !

!RestResourceHttpHandler methodsFor: 'request processing' stamp: 'cs 2/19/2009 17:53'!
process: anHTTPRequest 
	"Create a resource chain from the path of the http request. Fail if that chain is empty. Redirect processing according to used verb (POST, PUT, GET, DELETE)."

	| res |
	res := self getRequestChainFor: anHTTPRequest uri identifierPath.
	res isEmpty ifTrue: [^ self responseForFailure: anHTTPRequest].	
	anHTTPRequest isPostRequest ifTrue: [^ self processPost: res fields: anHTTPRequest postFields].
	anHTTPRequest isGetRequest ifTrue: [^ self processGet: res].
	anHTTPRequest isPutRequest ifTrue: [^ self processPut: res fields: anHTTPRequest putFields].
	anHTTPRequest isDeleteRequest ifTrue: [^ self processDelete: res].! !

!RestResourceHttpHandler methodsFor: 'request processing' stamp: 'mt 2/16/2009 16:59'!
processDelete: aResourceChain 

	|resClass id|
	resClass := aResourceChain first key.
	id := aResourceChain first value.
	^ self delete: resClass id: id! !

!RestResourceHttpHandler methodsFor: 'request processing' stamp: 'mt 2/16/2009 16:57'!
processGet: aResourceChain 

	|resClass id|
	resClass := aResourceChain first key.
	id := aResourceChain first value.
	
	id isNil
		ifTrue: [^ self read: resClass]
		ifFalse:[^ self read: resClass id: id].! !

!RestResourceHttpHandler methodsFor: 'request processing' stamp: 'mt 2/16/2009 16:56'!
processPost: aResourceChain fields: someFields 
	
	|resClass|
	resClass := aResourceChain first key.
	^ self create: resClass from: someFields! !

!RestResourceHttpHandler methodsFor: 'request processing' stamp: 'mt 2/16/2009 16:58'!
processPut: aResourceChain fields: someFields 

	|resClass id|
	resClass := aResourceChain first key.
	id := aResourceChain first value.
	^ self update: resClass from: someFields id: id! !

!RestResourceHttpHandler methodsFor: 'http-get-helper' stamp: 'cs 2/19/2009 21:15'!
read: resClass
	"Read information about all entities of a given class. Works only for projects or users."
	
	| response |
	response := resClass all collect: [:entity|
		Dictionary newFrom: (
			resClass allInstVarNames collect: [:name| name -> (entity instVarNamed: name)])].
	^ HTTPResponse ok entity: (Json render: response).! !

!RestResourceHttpHandler methodsFor: 'http-get-helper' stamp: 'cs 2/19/2009 21:15'!
read: resClass id: id
	"Read information about one special resource with the given id. Works only for projects and users."

	| entity response |
	entity := resClass all
		detect: [:each | each id = id]
		ifNone: [^ self responseForNotFound: resClass id: id].
	response := Dictionary new.
	resClass allInstVarNames do:
		[:name| response at: name put: (entity instVarNamed: name)].
	^ HTTPResponse ok entity: (Json render: response).! !

!RestResourceHttpHandler methodsFor: 'private' stamp: 'mt 2/16/2009 17:35'!
resolveName: aClass
	"Inverted lookup in the pool of registered resourcen."

	resources keysAndValuesDo: [:key :value |
		value = aClass ifTrue: [^ key]].
	^ nil.! !

!RestResourceHttpHandler methodsFor: 'private' stamp: 'mt 2/16/2009 16:45'!
resolveResource: aResourceName ifAbsent: aBlock
	"Try to resolve the given resource name to a key in the registered resource pool. Plural will be converted to singular: userstories -> userstory, projects -> project. The resource name has to be lowercase."

	| resourceKey |
	resourceKey := aResourceName.
	(resourceKey endsWith: 'ies')
		ifTrue:[resourceKey := (resourceKey truncateTo: resourceKey size - 3), 'y'].
	(resourceKey endsWith: 's')
		ifTrue:[resourceKey := resourceKey truncateTo: resourceKey size - 1].
		
	^ resources at: resourceKey ifAbsent: aBlock.! !

!RestResourceHttpHandler methodsFor: 'private' stamp: 'mt 2/16/2009 17:37'!
responseForFailure: anHttpRequest 
	^ nil asHttpResponseTo: anHttpRequest! !

!RestResourceHttpHandler methodsFor: 'private' stamp: 'cs 2/19/2009 21:16'!
responseForNotFound: resource id: id
	^ HTTPResponse notFound
		entity:
			(self resolveName: resource capitalized),
			' with id ', id asString,
			' not found.'! !

!RestResourceHttpHandler methodsFor: 'http-put-helper' stamp: 'cs 2/19/2009 21:16'!
update: resClass from: fields id: id
	"Works only with projects and users."

	| entity |
	entity := resClass all
		detect: [:each | each id = id]
		ifNone: [^ self responseForNotFound: resClass id: id].
	fields keysAndValuesDo: [:key :value |
		entity instVarNamed: key asString put: value first].
	^ HTTPResponse ok entity: 'updated'! !

!HttpRequest methodsFor: '*rest-server' stamp: 'mt 2/18/2009 14:21'!
putFields
	"Answer the multipart fields for a post request (if in fact this is a
	PUT request"

	^self propertyAt: #putFields ifAbsentPut: [
		" parse PUT fields in the body of request, if it's standard form data "

		(self isPutRequest and: [ 
			self contentType = MIMEDocument contentTypeFormData 
		]) ifTrue: [
			self class
				decodeUrlEncodedForm: (stream next: self contentLength)
				multipleValues: true.
		] ifFalse: [
			Dictionary new
		].
	]! !
